'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _isInteger = require('babel-runtime/core-js/number/is-integer');

var _isInteger2 = _interopRequireDefault(_isInteger);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _errors = require('./errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * An immutable object which represents a card of a French deck.
 * @class Card
 */
var Card = function () {
  (0, _createClass3.default)(Card, [{
    key: 'rank',


    /**
     * Rank of the card, represented by its shorthand.
     */
    get: function get() {
      if (!this.cachedRank) {
        this.cachedRank = _config2.default.cardRanks[this.id % _config2.default.cardRanks.length];
      }

      return this.cachedRank;
    }

    /**
     * Suit of the card, represented by its shorthand.
     */

    /**
     * Numeric ID of the card.
     */

  }, {
    key: 'suit',
    get: function get() {
      if (!this.cachedSuit) {
        this.cachedSuit = _config2.default.cardSuits[Math.floor(this.id / _config2.default.cardRanks.length)];
      }

      return this.cachedSuit;
    }

    /**
     * @param {string|number} value Shorthand or numeric ID of the card.
     */

  }]);

  function Card(value) {
    (0, _classCallCheck3.default)(this, Card);

    if (typeof value === 'string') {
      if (value.length < 2) {
        throw new _errors.InvalidCardValueError(value);
      }

      var rank = value.substring(0, value.length - 1);
      var rankIndex = _config2.default.cardRanks.indexOf(rank);
      if (rankIndex < 0) {
        throw new _errors.InvalidCardValueError(value);
      }
      this.cachedRank = rank;

      var suit = value[value.length - 1];
      var suitIndex = _config2.default.cardSuits.indexOf(suit);
      if (suitIndex < 0) {
        throw new _errors.InvalidCardValueError(value);
      }
      this.cachedSuit = suit;

      this.id = rankIndex + suitIndex * _config2.default.cardRanks.length;
    } else if ((0, _isInteger2.default)(value)) {
      if (value < 0 || value >= _config2.default.cardsInDeck) {
        throw new _errors.InvalidCardValueError(value);
      }

      this.id = value;
    } else {
      throw new _errors.InvalidCardValueError(value);
    }
  }

  (0, _createClass3.default)(Card, [{
    key: 'toString',
    value: function toString() {
      return this.rank + this.suit;
    }
  }]);
  return Card;
}();

exports.default = Card;
//# sourceMappingURL=card.js.map