'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _utils = require('./utils');

var Utils = _interopRequireWildcard(_utils);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * An immutable object which represents a deck of cards.
 * @class Deck
 */
var Deck = function () {

  /**
   * @param {Point[]} points Points of the deck.
   */
  function Deck(points) {
    (0, _classCallCheck3.default)(this, Deck);

    this.points = points;
  }

  /**
   * Encrypts all of the deck's points with the given secret.
   * @param {BigInt} secret Secret to encrypt with.
   * @returns {Deck}
   */

  /**
   * Points of the deck.
   */


  (0, _createClass3.default)(Deck, [{
    key: 'encrypt',
    value: function encrypt(secret) {
      var bi = secret.fromRed();
      return new Deck(this.points.map(function (point) {
        return point.mul(bi);
      }));
    }

    /**
     * Decrypts all of the deck's points with the given secret.
     * @param {BigInt} secret Secret to be used for decryption.
     * @returns {Deck}
     */

  }, {
    key: 'decrypt',
    value: function decrypt(secret) {
      var bi = secret.invm(_config2.default.ec.n);
      return new Deck(this.points.map(function (point) {
        return point.mul(bi);
      }));
    }

    /**
     * Shuffles all of the deck's points.
     * @returns {Deck}
     */

  }, {
    key: 'shuffle',
    value: function shuffle() {
      return new Deck(Utils.shuffleArray(this.points));
    }

    /**
     * Locks all of the deck's points with the given secrets.
     * @param {BigInt[]} secrets Secrets to lock with.
     * @returns {Deck}
     */

  }, {
    key: 'lock',
    value: function lock(secrets) {
      return new Deck(this.points.map(function (point, i) {
        return point.mul(secrets[i].fromRed());
      }));
    }

    /**
     * Unlocks a single point by using multiple secrets.
     * @param {number} index Index of the card to be unlocked.
     * @param {BigInt[]} secrets Secrets to be used for unlocking.
     * @returns {Point}
     */

  }, {
    key: 'unlockSingle',
    value: function unlockSingle(index, secrets) {
      var point = this.points[index];

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(secrets), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var secret = _step.value;

          point = point.mul(secret.invm(_config2.default.ec.n));
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return point;
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      return {
        points: this.points.map(function (point) {
          return Utils.pointToJSON(point);
        })
      };
    }
  }]);
  return Deck;
}();

exports.default = Deck;
//# sourceMappingURL=deck.js.map