'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _pokersolver = require('pokersolver');

var _bet = require('./bet');

var _bet2 = _interopRequireDefault(_bet);

var _card = require('./card');

var _card2 = _interopRequireDefault(_card);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _deck = require('./deck');

var _deck2 = _interopRequireDefault(_deck);

var _gameState = require('./enums/game-state');

var _gameState2 = _interopRequireDefault(_gameState);

var _player = require('./player');

var _player2 = _interopRequireDefault(_player);

var _utils = require('./utils');

var Utils = _interopRequireWildcard(_utils);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A mutable object which serves as an entry point for creating mental poker
 * games.
 * @class Game
 */
var Game = function () {

  /**
   * @param {?Object} params Parameters to be assigned to the new instance.
   */


  /**
   * Keeps an ordered list of unpickable (owned or opened) card indexes.
   */


  /**
   * Index of the currently acting player in the turn. `-1` if the game has
   * ended.
   */


  /**
   * Ordered list of players of the game.
   */


  /**
   * Player object of self.
   */
  function Game(params) {
    (0, _classCallCheck3.default)(this, Game);
    this.state = _gameState2.default.GENERATING_INITIAL_DECK;
    this.actingPlayerIndex = 0;
    this.deckSequence = [];
    this.unpickableCardIndexes = [];
    this.cardsOfCommunity = [];

    (0, _assign2.default)(this, params);

    if (!this.playerSelf) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(this.players), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var player = _step.value;

          // The player whose secrets are known should be self
          var isSelf = true;
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = (0, _getIterator3.default)(player.secrets), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var secret = _step2.value;

              if (!secret) {
                isSelf = false;
                break;
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }

          if (isSelf) {
            this.playerSelf = player;
            break;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }

  /**
   * Returns the currently acting player in the turn.
   * @returns {Player}
   */


  /**
   * Keeps an ordered list of community cards.
   */


  /**
   * Keeps an ordered list of decks used throughout the game, allowing easy
   * verification at the end of the game.
   */

  /**
   * Represents the current state of the game.
   */


  (0, _createClass3.default)(Game, [{
    key: 'getPickableCardIndexes',


    /**
     * Returns all card indexes which are not yet owned or opened by anyone.
     * @returns {number[]}
     */
    value: function getPickableCardIndexes() {
      var _this = this;

      return (0, _from2.default)(new Array(_config2.default.cardsInDeck), function (v, i) {
        return i;
      }).filter(function (v) {
        return _this.unpickableCardIndexes.indexOf(v) < 0;
      });
    }

    /**
     * Returns a random pickable card index.
     * @returns {number}
     */

  }, {
    key: 'getRandomPickableCardIndex',
    value: function getRandomPickableCardIndex() {
      var pickableCardIndexes = this.getPickableCardIndexes();

      // Return the index of a pickable card
      return pickableCardIndexes[Utils.getRandomInt(0, pickableCardIndexes.length)];
    }

    /**
     * Generates the initial deck of the game's deck sequence. By default, points
     * of players will be used for generating deck points. If any player-generated
     * point is missing, then deck points will be generated at random.
     * @returns {Game}
     */

  }, {
    key: 'generateInitialDeck',
    value: function generateInitialDeck() {
      // Try generating points by combining the points of players
      var deckPoints = new Array(_config2.default.cardsInDeck);
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = (0, _getIterator3.default)(this.players), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var playerPoints = _step3.value.points;

          // On failure, generate deck points at random
          if (playerPoints.length !== _config2.default.cardsInDeck) {
            deckPoints = Utils.getRandomPoints();
            break;
          }

          for (var i = playerPoints.length - 1; i >= 0; --i) {
            var playerPoint = playerPoints[i];
            var deckPoint = deckPoints[i];

            // Add the player's current point to the corresponding deck point
            deckPoints[i] = deckPoint ? deckPoint.add(playerPoint) : playerPoint;
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      this.deckSequence = [new _deck2.default(deckPoints)];
      this.state = _gameState2.default.SHUFFLING_DECK;
      return this;
    }

    /**
     * Shuffles the deck using the secrets of self, and on request, adds it to the
     * deck sequence of the game.
     * @param {Player} [player] Player object to shuffle the deck with. Defaults
     * to the player object of self.
     * @param {boolean} [isAddableToSequence=true] True whether the result should
     * be added to the game's deck sequence on success.
     * @param {Deck} [deck] Deck to be shuffled. If omitted, then uses the last
     * deck in the game's deck sequence.
     * @returns {?Deck} Null if an invalid parameter was specified.
     */

  }, {
    key: 'shuffleDeck',
    value: function shuffleDeck() {
      var player = arguments.length <= 0 || arguments[0] === undefined ? this.playerSelf : arguments[0];
      var isAddableToSequence = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];
      var deck = arguments.length <= 2 || arguments[2] === undefined ? this.deckSequence[this.deckSequence.length - 1] : arguments[2];

      if (!deck) return null;

      // Improve the accessibility of secrets later by using the last one now
      var lastSecret = player.secrets[player.secrets.length - 1];

      // Shuffle the deck and then encrypt it to avoid data leaks
      var nextDeck = deck.shuffle().encrypt(lastSecret);
      if (isAddableToSequence) {
        this.addDeckToSequence(nextDeck);
      }
      return nextDeck;
    }

    /**
     * Locks the deck using the secrets of self, and on request, adds it to the
     * deck sequence of the game.
     * @param {Player} [player] Player object to lock the deck with. Defaults to
     * the player object of self.
     * @param {boolean} [isAddableToSequence=true] True whether the result should
     * be added to the game's deck sequence on success.
     * @param {Deck} [deck] Deck to be locked. If omitted, then uses the last deck
     * in the game's deck sequence.
     * @returns {?Deck} Null if an invalid parameter was specified.
     */

  }, {
    key: 'lockDeck',
    value: function lockDeck() {
      var player = arguments.length <= 0 || arguments[0] === undefined ? this.playerSelf : arguments[0];
      var isAddableToSequence = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];
      var deck = arguments.length <= 2 || arguments[2] === undefined ? this.deckSequence[this.deckSequence.length - 1] : arguments[2];

      if (!deck) return null;

      var lastSecret = player.secrets[player.secrets.length - 1];

      // Remove the shuffle encryption and then lock each card one by one
      var nextDeck = deck.decrypt(lastSecret).lock(player.secrets);
      if (isAddableToSequence) {
        this.addDeckToSequence(nextDeck);
      }
      return nextDeck;
    }

    /**
     * Adds a shuffled or locked deck to the game's deck sequence. Automatically
     * takes turn on behalf of the currently acting player, and updates game state
     * if necessary.
     * @param {Deck} deck Deck to be added to the game's deck sequence.
     * @returns {boolean} True if the action was successful.
     */

  }, {
    key: 'addDeckToSequence',
    value: function addDeckToSequence(deck) {
      // Disallow modifying deck sequence if the game has already started
      if (this.state >= _gameState2.default.PLAYING) return false;

      this.deckSequence.push(deck);

      // Update game state if the turn has come to an end
      if (this.takeTurn() === 0) {
        this.state = this.state === _gameState2.default.SHUFFLING_DECK ? _gameState2.default.LOCKING_DECK : _gameState2.default.PLAYING;
      }

      return true;
    }

    /**
     * Takes turn on behalf of the currently acting player, updating
     * `actingPlayerIndex` with the next value in its cycle.
     * @param {?Bet} [bet] Bet made by the currently acting player.
     * @returns {number} Index of the next player in turn.
     */

  }, {
    key: 'takeTurn',
    value: function takeTurn(bet) {
      if (bet) {
        this.actingPlayer.bets.push(bet);
      }

      // Check whether only 1 player is left in the game
      if (this.players.filter(function (player) {
        return !player.hasFolded;
      }).length === 1) {
        // End the game immediately
        this.end();
      } else {
        // Advance to the next player who hasn't folded
        do {
          this.actingPlayerIndex = (this.actingPlayerIndex + 1) % this.players.length;
        } while (this.actingPlayer.hasFolded);
      }

      return this.actingPlayerIndex;
    }

    /**
     * Picks an unowned card at the given index, unlocking it by its corresponding
     * secrets.
     * @param {number} index Index of the card to be picked.
     * @param {boolean} [isMadeUnpickable=true] Determines whether the picked card
     * should be made unpickable on success.
     * @returns {?Card} On success, an instance of the picked card. Otherwise (if
     * any of the necessary secrets are unknown or the card at the given index has
     * already been drawn), null.
     */

  }, {
    key: 'pickCard',
    value: function pickCard(index) {
      var isMadeUnpickable = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

      if (this.unpickableCardIndexes.indexOf(index) < 0) {
        // Gather each player's secret at the given index
        var secrets = this.players.map(function (player) {
          return player.secrets[index];
        });

        var currentDeck = this.deckSequence[this.deckSequence.length - 1];
        var pointUnlocked = currentDeck.unlockSingle(index, secrets);
        var initialDeckPoints = this.deckSequence[0].points;

        for (var i = initialDeckPoints.length - 1; i >= 0; --i) {
          if (initialDeckPoints[i].eq(pointUnlocked)) {
            // Make the unlocked card unpickable if necessary
            if (isMadeUnpickable) {
              this.unpickableCardIndexes.push(index);
            }

            return new _card2.default(i);
          }
        }
      }

      return null;
    }

    /**
     * Picks an unowned card at the given index, and then draws it to the hand of
     * self.
     * @param {number} index Index of the card to be drawn.
     * @returns {?Card} On success, an instance of the drawn card. Otherwise (if
     * any of the necessary secrets are unknown or the card at the given index has
     * already been drawn), null.
     */

  }, {
    key: 'drawCard',
    value: function drawCard(index) {
      var card = this.pickCard(index);
      if (card) {
        this.playerSelf.cardsInHand.push(card);
      }

      return card;
    }

    /**
     * Picks an unowned card at the given index, and then opens it as a community
     * card.
     * @param {number} index Index of the card to be opened.
     * @returns {?Card} On success, an instance of the opened card. Otherwise (if
     * any of the necessary secrets are unknown or the card at the given index has
     * already been drawn), null.
     */

  }, {
    key: 'openCard',
    value: function openCard(index) {
      var card = this.pickCard(index);
      if (card) {
        this.cardsOfCommunity.push(card);
      }

      return card;
    }

    /**
     * Ends the game immediately, making no more player action possible.
     */

  }, {
    key: 'end',
    value: function end() {
      this.actingPlayerIndex = -1;
      this.state = _gameState2.default.ENDED;
    }

    /**
     * Verifies the entire game, looking for players who were not playing fairly.
     * @returns {Player[]} List of unfair players.
     */

  }, {
    key: 'verify',
    value: function verify() {
      var result = [];
      for (var i = this.players.length - 1; i >= 0; --i) {
        var player = this.players[i];

        if (
        // Check for deck shuffling mistakes
        !Utils.isArrayEqualWith(Utils.sortPoints([].concat((0, _toConsumableArray3.default)(this.shuffleDeck(player, false, this.deckSequence[i]).points))), Utils.sortPoints([].concat((0, _toConsumableArray3.default)(this.deckSequence[i + 1].points))), function (p1, p2) {
          return p1.eq(p2);
        }) ||

        // Check for deck locking mistakes
        !Utils.isArrayEqualWith(this.lockDeck(player, false, this.deckSequence[this.players.length + i]).points, this.deckSequence[this.players.length + i + 1].points, function (p1, p2) {
          return p1.eq(p2);
        })) {
          result.push(player);
        }
      }

      return result;
    }

    /**
     * Evaluates the hands of players, looking for the winner(s) of the game.
     * @param {string} [gameType=Config.gameType] Type of the game to evaluate
     * hands for.
     * @returns {Player[]} List of players who won the game.
     */

  }, {
    key: 'evaluateHands',
    value: function evaluateHands() {
      var gameType = arguments.length <= 0 || arguments[0] === undefined ? _config2.default.gameType : arguments[0];

      var pokerSolverGame = new _pokersolver.Game(gameType);
      var commonCardStrings = this.cardsOfCommunity.map(function (card) {
        return card.toString();
      });

      // Evaluate the hand of players who haven't folded
      var handsOfPlayers = new _map2.default();
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = (0, _getIterator3.default)(this.players), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var player = _step4.value;

          if (!player.hasFolded) {
            handsOfPlayers.set(_pokersolver.Hand.solve([].concat((0, _toConsumableArray3.default)(commonCardStrings), (0, _toConsumableArray3.default)(player.cardsInHand.map(function (card) {
              return card.toString();
            }))), pokerSolverGame), player);
          }
        }

        // Look for winner hands and map them to their owners
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      return _pokersolver.Hand.winners([].concat((0, _toConsumableArray3.default)(handsOfPlayers.keys()))).map(function (hand) {
        return handsOfPlayers.get(hand);
      });
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      return (0, _extends3.default)({
        state: _gameState2.default.toString(this.state),
        players: this.players.map(function (player) {
          return player.toJSON();
        }),
        actingPlayerIndex: this.actingPlayerIndex
      }, this.deckSequence[0] && this.deckSequence[0].toJSON());
    }
  }, {
    key: 'actingPlayer',
    get: function get() {
      return this.players[this.actingPlayerIndex];
    }
  }]);
  return Game;
}();

exports.default = Game;
//# sourceMappingURL=game.js.map