'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _bet = require('./bet');

var _bet2 = _interopRequireDefault(_bet);

var _card = require('./card');

var _card2 = _interopRequireDefault(_card);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _betType = require('./enums/bet-type');

var _betType2 = _interopRequireDefault(_betType);

var _utils = require('./utils');

var Utils = _interopRequireWildcard(_utils);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A mutable object which represents a player of a game.
 * @class Player
 */
var Player = function () {

  /**
   * @param {?Object} params Parameters to be assigned to the new instance.
   */


  /**
   * Bets made by the player.
   */


  /**
   * Secrets of the player. Must be kept hidden from others until the end of the
   * game and shall not be modified directly.
   */

  /**
   * Public key of the player.
   */
  function Player(params) {
    (0, _classCallCheck3.default)(this, Player);
    this.points = [];
    this.secrets = new Array(_config2.default.cardsInDeck + 1);
    this.secretHashes = [];
    this.bets = [];
    this.cardsInHand = [];

    (0, _assign2.default)(this, params);

    // Force setting `secretHashes` if all the secrets are known
    if (this.secretHashes.length === 0) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(this.secrets), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var secret = _step.value;

          if (!secret) return;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      this.secretHashes = this.secrets.map(function (secret) {
        return Utils.getSecretHash(secret);
      });
    }
  }

  /**
   * Returns true whether the player has folded.
   * @returns {boolean}
   */


  /**
   * List of cards which are in the hand of the player.
   */


  /**
   * Secret hashes of the player. Used for secret verification at the end of the
   * game.
   */


  /**
   * Points generated by the player.
   */


  (0, _createClass3.default)(Player, [{
    key: 'addSecret',


    /**
     * Adds and verifies a secret at the given index.
     * @param {number} index Index of the secret to be added.
     * @param {BigInt} secret Secret to be added.
     * @returns {boolean} False whether verification has failed, otherwise, true.
     */
    value: function addSecret(index, secret) {
      // Avoid re-addition of secrets
      if (this.secrets[index]) return true;

      // Check whether the given secret satisfies its corresponding hash
      if (Utils.getSecretHash(secret) !== this.secretHashes[index]) {
        return false;
      }

      this.secrets[index] = secret;
      return true;
    }

    /**
     * Generates random points for the player.
     * @returns {Player}
     */

  }, {
    key: 'generatePoints',
    value: function generatePoints() {
      this.points = Utils.getRandomPoints();
      return this;
    }

    /**
     * Generates random secrets and their corresponding hashes for the player.
     * @returns {Player}
     */

  }, {
    key: 'generateSecrets',
    value: function generateSecrets() {
      this.secrets = Utils.getRandomSecrets();
      this.secretHashes = this.secrets.map(function (secret) {
        return Utils.getSecretHash(secret);
      });
      return this;
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      return (0, _extends3.default)({}, this.publicKey && { publicKey: this.publicKey }, this.points.length > 0 && {
        points: this.points.map(function (point) {
          return Utils.pointToJSON(point);
        })
      }, this.secretHashes.length > 0 && { secretHashes: this.secretHashes });
    }
  }, {
    key: 'hasFolded',
    get: function get() {
      if (this.bets.length === 0) return false;

      return this.bets[this.bets.length - 1].type === _betType2.default.FOLD;
    }
  }]);
  return Player;
}();

exports.default = Player;
//# sourceMappingURL=player.js.map